<!DOCTYPE html>
{% load i18n %}
<html>
    <head>
        <title>{% trans "无访问权限" %}</title>
    </head>
    <body>
        <div class="no-permission-exception" style="text-align: center;">
            <img src="https://staticfile.qq.com/TencentBlueKing/bkui-vue3/77ca9903/assets/403-1dd24f60.svg" alt="无应用访问权限" style="width: 480px; height: 240px;" />
            <p style="margin: 18px 0 0; font-size: 24px; line-height: 32px; color: #313238;">无该应用访问权限</p>
            <p style="margin: 16px 0 0; font-size: 14px; line-height: 22px; color: #63656e;">{{ hint_msg }}</p>
        </div>
    </body>
</html>
